class NotUpdateToken(Exception):
    def __init__(self, message = 'Не удалось обновить токен'):
        self.message = message
        super().__init__(self.message)

class NotFoundError(Exception):
    def __init__(self, message = 'Не удалось скачать файл'):
        self.message = message
        super().__init__(self.message)


from loguru import logger
import os

log_dir = "logs"
os.makedirs(log_dir, exist_ok=True)
logger.remove()
logger.add(
    os.path.join(log_dir, "app.log"),
    rotation = '10 MB',
    retention = '5 days',
    level = "DEBUG",
    enqueue=True,
    mode='w',
)


import json
import os

# Распаковка координат полигона
def read_polygon(filename):
    with open(filename, 'r', encoding='utf-8') as file:
        geojson_data = json.load(file)
    feature = geojson_data["features"][0]  # Первый объект
    geometry = feature["geometry"]  # Геометрия первого объекта

    if geometry["type"] != "Polygon":
        raise ValueError("Геометрия не является Polygon.")
    file_polygon_coords = geometry["coordinates"][0]
    # Не свапаем при чтении для правильного запроса координат
    # Возвращаем координаты внешнего контура
    return file_polygon_coords

def coords_to_str(polygon_coords):
    # Форматируем каждую пару координат в "latitude longitude"
    formatted_coords = ["{:.8f} {:.8f}".format(lon, lat) for lon, lat in polygon_coords] # не свапаем для ODATA
    formatted_str_coords = f"({','.join(formatted_coords)})"
    return formatted_str_coords

def swap_polygon_coords(polygon_coords):
    swapped_lat_lon = [[point[1], point[0]] for point in polygon_coords]
    # Возвращаем координаты внешнего контура
    return swapped_lat_lon

def get_polygon_center(polygon_coords):
    total_lon = sum(point[0] for point in polygon_coords)
    total_lat = sum(point[1] for point in polygon_coords)
    n = len(polygon_coords)
    centroid_lon = total_lon / n
    centroid_lat = total_lat / n
    return centroid_lon, centroid_lat


def fix_coordinates(geojson_data):
    """
    Исправляет координаты, удаляя 360 из значений долготы.
    """
    for feature in geojson_data['features']:
        if feature['geometry']['type'] == 'Polygon':
            for polygon in feature['geometry']['coordinates']:
                for i, coord in enumerate(polygon):
                    while coord[0] < -180 or coord[0] > 180:
                        # Удаляем 360 из долготы
                        if coord[0] < -180:
                            coord[0] += 360
                        elif coord[0] > 180:
                            coord[0] -= 360
    return geojson_data


def save_fixed_geojson(input_file):
    """
    Читает GeoJSON из файла, исправляет координаты и сохраняет в новый файл.
    """
    output_file = os.path.join(os.path.dirname(input_file), "fixed_" + os.path.basename(input_file))
    with open(input_file, 'r', encoding='utf-8') as f:
        geojson_data = json.load(f)

    fixed_geojson = fix_coordinates(geojson_data)

    with open(output_file, 'w', encoding='utf-8') as f:
        json.dump(fixed_geojson, f, indent=4)


def correct_polygon_files_in_folder(input_folder):
    """
    Обходит все файлы в папке, находит GeoJSON файлы и обрабатывает их.
    """
    # Создаем выходную папку, если она не существует
    output_folder = input_folder
    if not os.path.exists(output_folder):
        os.makedirs(output_folder)

    # Обходим все файлы в папке
    for filename in os.listdir(input_folder):
        # Проверяем, что файл имеет расширение .geojson
        if filename.endswith('.geojson'):
            input_file = os.path.join(input_folder, filename)
            output_file = os.path.join(output_folder, f"fixed_{filename}")

            # Обрабатываем файл
            save_fixed_geojson(input_file)
            print(f"Обработан файл: {filename} -> {output_file}")


if __name__ == '__main__':
    folder = "../polygon_files/"
    correct_polygon_files_in_folder(folder)




import os
import sys
import requests
from dotenv import load_dotenv
from pathlib import Path


class TokenManager:
    def __init__(self, env_path:Path = ".env"):
        """
        Инициализация менеджера токенов.
        :param env_path: Путь к .env файлу.
        """
        self.env_path = env_path
        self.load_env()
        self.url = os.getenv("TOKEN_URL")
        self.username = os.getenv("USERNAME_COPERNICUS")
        self.password = os.getenv("PASSWORD_COPERNICUS")
        self.client_id = "cdse-public"

    def load_env(self):
        """
        Загрузка переменных окружения из .env файла.
        """
        if not os.path.exists(self.env_path):
            raise FileNotFoundError(f"Файл {self.env_path} не найден.")
        load_dotenv(self.env_path)

    def get_tokens(self):
        """
        Получение access_token и refresh_token через аутентификацию.
        """
        data = {
            "grant_type": "password",
            "username": self.username,
            "password": self.password,
            "client_id": self.client_id,
        }
        headers = {"Content-Type": "application/x-www-form-urlencoded"}
        response = requests.post(self.url, data=data, headers=headers)

        if response.status_code == 200:
            tokens = response.json()
            access_token = tokens.get("access_token")
            refresh_token = tokens.get("refresh_token")
            self.save_tokens(access_token, refresh_token)
            print("Токены успешно получены и сохранены.")
            return access_token
        else:
            print(f"Ошибка Аутентификации. Status Code: {response.status_code}. Response: {response.text}")
            sys.exit(1)

    def save_tokens(self, access_token, refresh_token):
        """
        Сохранение токенов в .env файл.
        """
        env_vars = {}
        if os.path.exists(self.env_path):
            with open(self.env_path, "r") as file:
                for line in file:
                    if "=" in line:
                        key, value = line.strip().split("=", 1)
                        env_vars[key] = value

        env_vars["ACCESS_TOKEN"] = str(access_token)
        env_vars["REFRESH_TOKEN"] = str(refresh_token)

        with open(self.env_path, "w") as file:
            for key, value in env_vars.items():
                file.write(f"{key}={value}\n")
        print(f"Токены успешно записаны в {self.env_path} файл.")

    def refresh_access_token(self):
        """
        Обновление access_token через refresh_token.
        """
        refresh_token = os.getenv("REFRESH_TOKEN")
        if not refresh_token:
            print("REFRESH_TOKEN не найден. Требуется переаутентификация.")
            return self.get_tokens()

        data = {
            "grant_type": "refresh_token",
            "refresh_token": refresh_token,
            "client_id": self.client_id,
        }
        headers = {"Content-Type": "application/x-www-form-urlencoded"}
        response = requests.post(self.url, data=data, headers=headers)

        if response.status_code == 200:
            new_access_token = response.json().get("access_token")
            self.save_tokens(new_access_token, refresh_token)  # Сохраняем только новый access_token
            print("ACCESS_TOKEN успешно обновлен.")
            self.load_env()
            return new_access_token
        else:
            print(f"Не удалось обновить ACCESS_TOKEN через REFRESH_TOKEN. Требуется переаутентификация."
                  f" Status Code: {response.status_code} Response:, {response.text}")
            return self.get_tokens()  # Переаутентификация, если refresh_token устарел

    def check_access_token(self):
        """
        Проверка работоспособности access_token.
        """
        access_token = os.getenv("ACCESS_TOKEN")
        if not access_token:
            print("ACCESS_TOKEN не найден. Требуется аутентификация.")
            return False

        url = "https://catalogue.dataspace.copernicus.eu/odata/v1/Products"
        headers = {"Authorization": f"Bearer {access_token}"}
        response = requests.get(url, headers=headers)

        if response.status_code == 200:
            return True
        else:
            print("ACCESS_TOKEN недействителен. Требуется обновление.")
            return False

    def get_valid_access_token(self):
        """
        Получение действительного access_token.
        """
        if self.check_access_token():
            print("ACCESS_TOKEN действителен")
            return os.getenv("ACCESS_TOKEN")
        else:
            print("Обновляем ACCESS_TOKEN...")
            return self.refresh_access_token()


import rasterio
from rasterio.windows import Window
import os
from tqdm import tqdm


def split_on_blocks(product_folder, block_size = 512):
    image_path = os.path.join(product_folder, 'output.tif')

    # Создаем папку для сохранения блоков
    output_folder = os.path.join(product_folder, 'output_blocks')
    os.makedirs(output_folder, exist_ok=True)
    # Открываем файл с помощью rasterio
    with rasterio.open(image_path) as src:
        # Получаем размеры изображения
        height, width = src.shape

        GREEN = "\033[92m"
        YELLOW = "\033[93m"
        BLUE = "\033[94m"
        RESET = "\033[0m"

        CUSTOM_BAR_FORMAT = (
            f"{GREEN}{{l_bar}}{RESET}"  # Левый символ (зеленый)
            f"{BLUE}{{bar}}{RESET}"  # Сам бар (синий)
            f"{GREEN}{{r_bar}}{RESET}"  # Правый символ (желтый)
        )
        total_blocks = ((height + block_size - 1) // block_size) * ((width + block_size - 1) // block_size)
        with tqdm(total=total_blocks,
                  desc=f"{GREEN}Разбиение на блоки{RESET}",
                  bar_format=CUSTOM_BAR_FORMAT,
                  unit="block",
                  ) as pbar:
            # Проходим по изображению и вырезаем блоки
            for i in range(0, height, block_size):
                for j in range(0, width, block_size):
                    # Определяем окно (блок)
                    window = Window(j, i, min(block_size, width - j), min(block_size, height - i))

                    # Читаем данные для текущего блока
                    block_data = src.read(window=window)

                    # Сохраняем блок в файл
                    block_path = os.path.join(output_folder, f'block_{i}_{j}.tif')
                    with rasterio.open(block_path, 'w',
                                       driver='GTiff',
                                       width=window.width,
                                       height=window.height,
                                       count=src.count,  # Количество каналов
                                       dtype=block_data.dtype,
                                       crs=src.crs,  # Система координат
                                       transform=src.window_transform(window)) as dst:
                        dst.write(block_data)
                    pbar.update(1)

    print(f"Изображение разбито на блоки и сохранено в папку '{output_folder}'.")

if __name__ == '__main__':
    product_folder = "../data/2025/01/19/09_12_19/product_files"
    split_on_blocks(product_folder)


import pytest
from pathlib import Path
from senphora.loader.token_manager import TokenManager
from dotenv import load_dotenv
import os
import requests_mock

@pytest.fixture
def env_path(tmp_path):
    """Фикстура для создания временного .env файла."""
    env_path = tmp_path / ".test.env"
    with open(env_path, "w") as f:
        f.write("TOKEN_URL=https://example.com/token\n")
        f.write("USERNAME_COPERNICUS=test_user\n")
        f.write("PASSWORD_COPERNICUS=test_password\n")
    return env_path

@pytest.fixture
def token_manager(env_path):
    """Фикстура для создания экземпляра TokenManager."""
    return TokenManager(env_path)

def test_load_env(token_manager):
    """Тест загрузки переменных окружения."""
    token_manager.load_env()
    assert token_manager.url == "https://example.com/token"
    assert token_manager.username == "test_user"
    assert token_manager.password == "test_password"

def test_get_tokens(token_manager, env_path):
    """Тест получения токенов."""
    with requests_mock.Mocker() as m:
        m.post(
            "https://example.com/token",
            json={"access_token": "test_access_token", "refresh_token": "test_refresh_token"},
            status_code=200,
        )

        access_token = token_manager.get_tokens()
        assert access_token == "test_access_token"

        # Проверяем, что токены записаны в .env файл
        load_dotenv(env_path)
        assert os.getenv("ACCESS_TOKEN") == "test_access_token"
        assert os.getenv("REFRESH_TOKEN") == "test_refresh_token"

def test_check_access_token(token_manager, env_path):
    """Тест проверки действительности access_token."""
    with requests_mock.Mocker() as m:
        m.get(
            "https://catalogue.dataspace.copernicus.eu/odata/v1/Products",
            status_code=200,
        )

        # Устанавливаем ACCESS_TOKEN в .env файл
        with open(env_path, "a") as f:
            f.write("ACCESS_TOKEN=test_access_token\n")

        assert token_manager.check_access_token()

def test_get_valid_access_token(token_manager, env_path):
    """Тест получения действительного access_token."""
    with requests_mock.Mocker() as m:
        m.get(
            "https://catalogue.dataspace.copernicus.eu/odata/v1/Products",
            status_code=401,  # Токен недействителен
        )
        m.post(
            "https://example.com/token",
            json={"access_token": "new_access_token"},
            status_code=200,
        )

        # Устанавливаем REFRESH_TOKEN в .env файл
        with open(env_path, "a") as f:
            f.write("REFRESH_TOKEN=test_refresh_token\n")

        access_token = token_manager.get_valid_access_token()
        assert access_token == "new_access_token"


import requests
from datetime import datetime
import pandas as pd
from ..polygon import coords_to_str

def products_by_polygon(polygon_coords:list, data_start:str, data_end:str,collection_name: str = "SENTINEL-2", top = 1000):
    polygon_coords = coords_to_str(polygon_coords)
    url = f"https://catalogue.dataspace.copernicus.eu/odata/v1/Products?$filter=Collection/Name eq '{collection_name}' and OData.CSC.Intersects(area=geography'SRID=4326;POLYGON({polygon_coords})') and ContentDate/Start gt {data_start} and ContentDate/Start lt {data_end}&$top={top}"
    json = requests.get(
        url).json()
    df = pd.DataFrame.from_dict(json['value'])
    pd.set_option('display.max_colwidth', None)
    print(f'Число результатов:{len(df)}')
    df_dict = df.to_dict()
    return df_dict

def products_by_coords(lat, lon, data_start, data_end, collection_name: str = "SENTINEL-2", top: int = 1000):
    url = f"https://catalogue.dataspace.copernicus.eu/odata/v1/Products?$filter=Collection/Name eq '{collection_name}' and OData.CSC.Intersects(area=geography'SRID=4326;POINT({lon} {lat})') and ContentDate/Start gt {data_start} and ContentDate/Start lt {data_end}&$top={top}"
    json = requests.get(
        url).json()
    df = pd.DataFrame.from_dict(json['value'])
    pd.set_option('display.max_colwidth', None)
    print(f'Число результатов:{len(df)}')
    df_dict = df.to_dict()
    return df_dict

# Вспомогательная функция для получения всех записей с нужным индексом
def get_data_by_index(data_dict, filtered_indexies):
    filtered_data = {}
    for key_name, value in data_dict.items():
        # print(f'{key_name} : {value}')
        filtered_data_items = {key: value for key, value in data_dict[key_name].items() if key in filtered_indexies}
        filtered_data[key_name] = filtered_data_items
    return filtered_data

def get_flat_data(data):
    # Определяем индекс (предполагаем, что все ключи имеют одинаковый индекс)
    index = next(iter(data.values())).keys().__iter__().__next__()

    # Преобразуем словарь, убирая индекс
    flat_data = {key: value[index] for key, value in data.items()}
    return flat_data


def filter_products_by_l2a(product_dict):
    filtered_products_id = {key: value for key, value in product_dict['Name'].items() if "L2A" in value}
    filtered_indecies = [key for key in filtered_products_id.keys()]

    filtered_products = get_data_by_index(product_dict, filtered_indecies)
    print(f"Число результатов l2a:{len(filtered_products['Name'])}" )
    return filtered_products


def filter_latest_product(product_dict):
    product_dates = product_dict['ContentDate']
    parsed_dates = {
        index: datetime.fromisoformat(value['Start'].rstrip('Z'))
        for index, value in product_dates.items()
    }
    latest_index = [max(parsed_dates, key=parsed_dates.get)]

    latest_product = get_data_by_index(product_dict, latest_index)
    flat_data = get_flat_data(latest_product)
    return flat_data


if __name__ == '__main__':
    lat = '31.52174'
    lon = '32.36212'
    collection_name = 'SENTINEL-2'
    data_start = '2025-01-01T00:00:00.000Z'
    data_end = '2025-01-20T00:00:00.000Z'
    product_dict = products_by_coords(lon=lon, lat=lat, collection_name=collection_name,
                                                    data_start=data_start, data_end=data_end)
    #Фильруем l2a
    l2a_products = filter_products_by_l2a(product_dict)

    # Фильтруем ближайший архив
    latest_product = filter_latest_product(l2a_products)
    print(latest_product['GeoFootprint'])



__all__ = [
    'read_polygon',
    'coords_to_str',
    'get_polygon_center',
    'swap_polygon_coords'
]
from .polygon_functions import read_polygon, coords_to_str, get_polygon_center, swap_polygon_coords


import glob
import os
import sys
import shutil
import hashlib

def find_channeles_files(picture_path):
    patterns = {
        "red": "*B04_10m.jp2",  # Красный канал (B04)
        "green": "*B03_10m.jp2",  # Зеленый канал (B03)
        "blue": "*B02_10m.jp2",  # Синий канал (B02)
    }
    channels = {"red": None, "green": None, "blue": None}
    for channel, pattern in patterns.items():
        # Используем glob для поиска файлов по шаблону
        files = glob.glob(os.path.join(picture_path, pattern))

        # Если найден ровно один файл, сохраняем его путь
        if len(files) == 1:
            channels[channel] = files[0]
        elif len(files) > 1:
            print(f"Найдено несколько файлов для канала {channel}: {files}")
            sys.exit(1)
        else:
            print(f"Файл для канала {channel} не найден.")
            sys.exit(1)
    return channels['red'], channels['green'], channels['blue']

def find_channeles_folder(product_path):
    r10m_pattern = os.path.join(product_path, "**", "IMG_DATA", "R10m")
    r10m_folders = glob.glob(r10m_pattern, recursive=True)

    if len(r10m_folders) == 0:
        print("Ошибка: Папка IMG_DATA/R10m не найдена.")
        print("Ищу просто папку IMG")
        rimg_pattern = os.path.join(product_path, "**", "IMG_DATA")
        rimg_folders = glob.glob(rimg_pattern, recursive=True)
        if len(rimg_folders) == 0:
            print("Ошибка: Папка IMG_DATA не найдена.")
        elif len(rimg_folders) > 1:
            print(f"Ошибка: Найдено несколько папок R10m: {r10m_folders}")
        else:
            rimg_folder = rimg_folders
            print(f"Найдены папки IMG: {rimg_folder}")
            return rimg_folder
    elif len(r10m_folders) > 1:
        print(f"Ошибка: Найдено несколько папок R10m: {r10m_folders}")
        return r10m_folders # если несколько папок, берём первую
    else:
        # Если найдена ровно одна папка R10m
        print(f"Найдена папка R10m: {r10m_folders}")
        return r10m_folders

def find_TCI_path(picture_path):
    # Проверяем, существует ли папка
    if not os.path.exists(picture_path):
        raise FileNotFoundError(f"Папка {picture_path} не существует.")

    # Проходим по всем файлам в папке
    for root, dirs, files in os.walk(picture_path):
        for file in files:
            # Проверяем, что файл имеет расширение .jp2 и содержит 'TCI' в названии
            if file.endswith(".jp2") and "TCI" in file:
                return os.path.join(root, file)  # Возвращаем полный путь к файлу

    # Если файл не найден, возвращаем None
    return None

def find_detections_png_files(data_dir):
    detection_pattern = os.path.join(data_dir, "**", "detections")
    detection_folders = glob.glob(detection_pattern, recursive=True)
    png_files = []
    for folder in detection_folders:
        if not os.path.exists(folder):
            print(f"Предупреждение: папка '{folder}' не существует.")
            continue
        png_files_in_folder = find_png_files_in_folder(folder)
        for png_file in png_files_in_folder:
            png_files.append(png_file)
        # for root, _, files in os.walk(folder):
        #     for file in files:
        #         # Проверяем, что файл имеет расширение .jp2 и содержит 'TCI' в названии
        #         if file.lower().endswith('.png'):  # Ищем файлы с расширением .png
        #             full_path = os.path.join(root, file)
        #             png_files.append(full_path)
    return png_files

def find_png_files_in_folder(png_dir):
    png_files = []
    for root, _, files in os.walk(png_dir):
        for file in files:
            # Проверяем, что файл имеет расширение .jp2 и содержит 'TCI' в названии
            if file.lower().endswith('.png'):  # Ищем файлы с расширением .png
                full_path = os.path.join(root, file)
                png_files.append(full_path)
    return png_files


def find_tiles_files(data_dir):
    tiles_pattern = os.path.join(data_dir, "**", "tiles")
    tiles_folders = glob.glob(tiles_pattern, recursive=True)
    jp2_files = []
    for folder in tiles_folders:
        if not os.path.exists(folder):
            print(f"Предупреждение: папка '{folder}' не существует.")
            continue
        jp2_files_in_folder = find_jp2_files_in_folder(folder)
        for jp2_file in jp2_files_in_folder:
            jp2_files.append(jp2_file)
    return jp2_files

def find_jp2_files_in_folder(jp2_dir):
    jp2_files = []
    for root, _, files in os.walk(jp2_dir):
        for file in files:
            # Проверяем, что файл имеет расширение .jp2 и содержит 'TCI' в названии
            if file.lower().endswith('.jp2'):  # Ищем файлы с расширением .png
                full_path = os.path.join(root, file)
                jp2_files.append(full_path)
    return jp2_files

def find_matching_jp2_files(png_file_paths, jp2_file_paths):
    png_names = {os.path.splitext(os.path.basename(path))[0] for path in png_file_paths}
    # Ищем .jp2-файлы с такими же именами
    matching_jp2_files = []
    for jp2_path in jp2_file_paths:
        jp2_name, _ = os.path.splitext(os.path.basename(jp2_path))
        if jp2_name in png_names:
            matching_jp2_files.append(jp2_path)
    return matching_jp2_files


def copy_file_to_folder(files_path, destination_folder):
    # Создаем целевую папку, если её нет
    if not os.path.exists(destination_folder):
        os.makedirs(destination_folder)

    for file_path in files_path:
        if not os.path.isfile(file_path):
            print(f"Предупреждение: файл '{file_path}' не существует или это не файл.")
            continue

        # Формируем имя файла в целевой папке
        file_name = os.path.basename(file_path)
        destination_path = os.path.join(destination_folder, file_name)

        try:
            shutil.copy2(file_path, destination_path)  # Используем copy2 для сохранения метаданных
        except Exception as e:
            print(f"Ошибка при копировании файла {file_path}: {e}")

def remove_duplicate_files(folder_path):
    """
    Удаляет дублирующиеся файлы в указанной папке.

    :param folder_path: Путь к папке, где нужно найти и удалить дубликаты.
    """
    hash_dict = {}  # Словарь для хранения хешей файлов

    # Функция для вычисления хеша файла
    def get_file_hash(file_path):
        hasher = hashlib.md5()
        with open(file_path, 'rb') as f:
            while chunk := f.read(8192):  # Читаем файл блоками по 8 КБ
                hasher.update(chunk)
        return hasher.hexdigest()

    # Обходим все файлы в папке
    for root, _, files in os.walk(folder_path):
        for file in files:
            file_path = os.path.join(root, file)

            try:
                file_hash = get_file_hash(file_path)  # Вычисляем хеш файла

                if file_hash in hash_dict:
                    print(f"Найден дубликат: {file_path}")
                    os.remove(file_path)  # Удаляем дубликат
                    print(f"Удален файл: {file_path}")
                else:
                    hash_dict[file_hash] = file_path  # Добавляем файл в словарь
            except Exception as e:
                print(f"Ошибка при обработке файла {file_path}: {e}")



import requests
import os
from datetime import datetime
from tqdm import tqdm
from ..utils.exceptions import NotFoundError

def load_product(access_token, product):
    url = f"https://download.dataspace.copernicus.eu/odata/v1/Products({product['Id']})/$value"
    headers = {"Authorization": f"Bearer {access_token}"}
    session = requests.Session()
    session.headers.update(headers)
    response = session.get(url, stream=True)
    # Check if the request was successful
    if response.status_code == 200:
        print("Получили добро на загрузку файла")
        date_str = product['ContentDate']['Start']
        date_obj = datetime.fromisoformat(date_str.rstrip('Z'))
        tile_name = get_tile_name(product['Name'])
        save_path = os.path.join(os.getcwd(),
            'data',
            str(date_obj.year),  # Год
            f"{date_obj.month:02d}",  # Месяц (с ведущим нулем)
            f"{date_obj.day:02d}", # День(с ведущим нулем)
            f"{tile_name}_{date_obj.hour:02d}_{date_obj.minute:02d}_{date_obj.second:02d}" #имя_тайла_Часы:минуты:секунды
        )
        # Выводим результат
        file_name = 'product.zip'
        file_path = os.path.join(save_path, file_name)
        print("Путь для сохранения файла:", file_path)
        # Создаем директории, если они не существуют
        os.makedirs(os.path.dirname(file_path), exist_ok=True)
        total_size = int(response.headers.get('content-length', 0))

        with open(file_path, "wb") as file:
            with tqdm(total=total_size, unit='B', unit_scale=True, desc=file_path) as pbar:
                for chunk in response.iter_content(chunk_size=8192):
                    if chunk:  # filter out keep-alive new chunks
                        file.write(chunk)
                        pbar.update(len(chunk))
        print("Загрузка завершена")
        return file_path, save_path
    else:
        print(f"Failed to download file. Status code: {response.status_code}")
        print(response.text)
        raise NotFoundError

def get_tile_name(name):
    strings = name.split("_")
    satellit_name = strings[0] + "_"
    tile = strings[5] + "_"
    processing_level = strings[1]
    full_name = [satellit_name, tile, processing_level]
    result = "".join(full_name)
    return result

if __name__ == "__main__":
    name = "S2B_MSIL2A_20250205T072009_N0511_R006_T39RTN_20250205T081649.SAFE"
    result = get_tile_name(name)
    print(result)



__all__ = [
    'unzip',
    'split_on_blocks',
    'formatted_current_time',
    'formatted_start_time',
    'get_coords_filename',
    'reformat_df_date'

]
from .unzipper import unzip
from .split_on_blocks import split_on_blocks
from .date_functions import formatted_current_time, formatted_start_time, get_coords_filename, reformat_df_date


__all__ = [
    'crop_image_by_polygon',
    'remove_black_borders',
    'split_jp2_into_tiles'
]

from .cropper import crop_image_by_polygon, remove_black_borders, split_jp2_into_tiles
from .format import resample


__all__ = [
    "TokenManager",
    "products_by_coords",
    "products_by_polygon",
    "load_product",
]

from .token_manager import TokenManager
from .find_product import products_by_coords, products_by_polygon
from .load_data import load_product


import os
from pathlib import Path
import rasterio
from PIL import Image
import numpy as np
from utils.find_folders import find_jp2_files_in_folder

def convert_all_files_in_folder(train_folder_path):
    folder_path = Path(train_folder_path)
    parent_path = folder_path.parent
    output_folder = os.path.join(parent_path, "png_data")
    os.makedirs(output_folder, exist_ok=True)
    jp2_files = find_jp2_files_in_folder(train_folder_path)
    for jp2_file in jp2_files:
        input_path = jp2_file
        file_name = os.path.basename(input_path)
        name_without_ext, ext = os.path.splitext(file_name)
        new_file_name = name_without_ext + ".png"
        output_path = os.path.join(output_folder, new_file_name)
        convert_jp2_to_png(input_path, output_path)
    return output_folder

def convert_jp2_to_png(input_path, output_path):
    """
    Преобразует файл .jp2 в файл .png без потери качества.

    :param input_path: Путь к входному файлу .jp2
    :param output_path: Путь к выходному файлу .png
    """
    try:
        # Открываем JP2 файл с помощью rasterio
        with rasterio.open(input_path) as src:
            # Читаем данные всех каналов
            data = src.read()

            # Проверяем количество каналов
            if data.shape[0] > 3:
                print(
                    "Внимание: Исходное изображение содержит более 3 каналов. Будут использованы только первые 3 (R, G, B).")
                data = data[:3]  # Выбираем только первые три канала (R, G, B)

            # Транспонируем данные для совместимости с Pillow (каналы должны быть последней осью)
            image_data = np.transpose(data, (1, 2, 0))

            # Конвертируем данные в формат uint8 (если они еще не в этом формате)
            if image_data.dtype != np.uint8:
                image_data = ((image_data - image_data.min()) / (image_data.max() - image_data.min()) * 255).astype(
                    np.uint8)

            # Создаем изображение с помощью Pillow
            image = Image.fromarray(image_data)

            # Сохраняем изображение в формате PNG
            image.save(output_path, format="PNG")

        # print(f"Файл успешно преобразован: {output_path}")

    except Exception as e:
        print(f"Ошибка при преобразовании файла: {e}")

if __name__ == "__main__":
    # Папка с тайлами
    train_folder = "../data/detections/matching_jp2/"
    jp2_files = find_jp2_files_in_folder(train_folder)
    print(len(jp2_files))
    # file_name = '../data/train_data/bare_data\\tile_-76.312922_37.017280.jp2'
    # output_path = '../data/train_data/png_data\\tile_-76.312922_37.017280.png'
    # convert_jp2_to_png(file_name, output_path)
    convert_all_files_in_folder(train_folder)



from datetime import datetime, timezone, timedelta
import pandas as pd


def formatted_current_time():
    # Получаем текущее время в UTC
    current_time = datetime.now(timezone.utc)

    # Форматируем время в требуемом формате
    formatted_time = current_time.strftime('%Y-%m-%dT%H:%M:%S.%f')[:-3] + 'Z'
    return formatted_time

def formatted_start_time(last_days: int = 1, last_hours: int = 0):
    current_time = datetime.now(timezone.utc)
    start_time = current_time - timedelta(days=last_days, hours=last_hours)
    formatted_time = start_time.strftime('%Y-%m-%dT%H:%M:%S.%f')[:-3] + 'Z'
    return formatted_time

def get_coords_filename():
    current_time = datetime.now(timezone.utc)
    formatted_time = current_time.strftime('%Y-%m-%dT%H:%M')
    return formatted_time

def reformat_df_date(df):
    # Преобразуем столбцы с датами в формат datetime
    df['OriginDate'] = pd.to_datetime(df['OriginDate'])
    df['PublicationDate'] = pd.to_datetime(df['PublicationDate'])
    df['ModificationDate'] = pd.to_datetime(df['ModificationDate'])

    # Форматируем даты в более наглядный вид
    df['OriginDate'] = df['OriginDate'].dt.strftime('%Y-%m-%d %H:%M:%S')
    df['PublicationDate'] = df['PublicationDate'].dt.strftime('%Y-%m-%d %H:%M:%S')
    df['ModificationDate'] = df['ModificationDate'].dt.strftime('%Y-%m-%d %H:%M:%S')

    # Обрабатываем столбец ContentDate
    df['ContentDate'] = df['ContentDate'].apply(
        lambda
            x: f"{pd.to_datetime(x['Start']).strftime('%Y-%m-%d %H:%M:%S')}" #- {pd.to_datetime(x['End']).strftime('%Y-%m-%d %H:%M:%S')}
    )
    return df


def calculate_date_difference(date1, date2):
    """
    Функция для расчета разницы между двумя датами в днях.

    :param date1: Первая дата (в формате 'YYYY-MM-DD')
    :param date2: Вторая дата (в формате 'YYYY-MM-DD')
    :return: Разница между датами в днях
    """
    try:
        # Преобразуем строки в объекты datetime
        date_format = "%Y-%m-%d"
        d1 = datetime.strptime(date1, date_format)
        d2 = datetime.strptime(date2, date_format)

        # Вычисляем разницу между датами
        difference = abs((d2 - d1).days)  # Используем abs() для получения положительной разницы
        return difference
    except ValueError as e:
        return f"Ошибка: Неверный формат даты. Пожалуйста, используйте формат 'YYYY-MM-DD'. Подробности: {e}"


import zipfile
import os
from tqdm import tqdm


def unzip(zip_file_path, save_path):
    # Path to the ZIP file
    # Directory to extract the files
    extract_dir = os.path.join(os.getcwd(), save_path, 'product_files')

    # Create the extraction directory if it doesn't exist
    if not os.path.exists(extract_dir):
        os.makedirs(extract_dir)

    # Unzip the file
    with zipfile.ZipFile(zip_file_path, 'r') as zip_ref:
        file_list = zip_ref.namelist()

        with tqdm(total=len(file_list), unit="file", desc="Extracting") as pbar:
            for file in file_list:
                zip_ref.extract(file, extract_dir)
                pbar.update(1)
    print("Распаковка завершена")




from setuptools import setup, find_packages

setup(
    name="senphora",  # Имя вашей библиотеки
    version="0.1.3",  # Версия
    packages=find_packages(where="src"),  # Автоматически находит все пакеты
    package_dir={"": "src"},
    install_requires=[
        "python-dotenv>=1.0.1",
        "requests>=2.32.3",
        "pandas>=2.2.3",
        "tqdm>=4.67.1",
        "rasterio>=1.4.3",
        "pyproj>=3.7.1",
        "shapely>=2.0.7",
    ],  # Зависимости (если есть)
    author="Babahasko",
    author_email="zakutni4ek@gmail.com",
    description="Senphora Python library for working with sentinel API",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    python_requires=">=3.12",
)



import rasterio
from rasterio.enums import Resampling

def resample(image_path):
    print(f"Ресэмплинг итогового изображения")
    with rasterio.open(image_path, 'r+') as dataset:
        overviews = [2, 4, 8, 16]  # Define the overview levels you want to create
        dataset.build_overviews(overviews, Resampling.average)
    print(f"Ресэмплинг завершен. Итоговое изображение пересохранено: {image_path}")


import os

import numpy as np
import rasterio
from tqdm import tqdm
from ..polygon import read_polygon
from pyproj import Transformer
from rasterio.crs import CRS
from rasterio.mask import mask
from rasterio.windows import Window
from shapely.geometry import Polygon, mapping
from ..utils.find_folders import find_channeles_folder, find_TCI_path
from .format import resample


def crop_image_by_polygon(product_folder, polygon_coords):
    r10m_folders = find_channeles_folder(product_folder)
    for r10m_folder in r10m_folders:
        input_image_path = find_TCI_path(r10m_folder)
        output_image_folder = os.path.join(product_folder, "cropped")
        file_name = 'cropped.jp2'
        output_image_path = os.path.join(output_image_folder, file_name)
        os.makedirs(os.path.dirname(output_image_path), exist_ok=True)
        # Преобразование координат полигона из EPSG:4326 в EPSG:32636
        with rasterio.open(input_image_path) as src:
            raster_crs = src.crs  # Получаем CRS растра (должно быть EPSG:32636)
            print(raster_crs)

            # Преобразование координат
            transformer = Transformer.from_crs(CRS.from_epsg(4326), raster_crs, always_xy=True)
            transformed_coords = [transformer.transform(x, y) for x, y in polygon_coords]
            polygon_geom = Polygon(transformed_coords)

            # Проверка пересечения полигонов
            raster_bounds = src.bounds
            if not Polygon.from_bounds(*raster_bounds).intersects(polygon_geom):
                raise ValueError("Полигон не пересекается с растровым изображением.")

            # Обрезка изображения по полигону
            geoms = [mapping(polygon_geom)]
            out_image, out_transform = mask(src, geoms, crop=True)

            # Копирование метаданных
            out_meta = src.meta.copy()
            out_meta.update({
                "driver": "GTiff",  # Формат выходного файла
                "height": out_image.shape[1],  # Высота обрезанного изображения
                "width": out_image.shape[2],  # Ширина обрезанного изображения
                "transform": out_transform,  # Новая матрица трансформации
                "count": src.count,  # Число каналов (должно быть 3 для RGB)
                "dtype": src.dtypes[0]  # Тип данных (например, uint8)
            })

        # Сохранение обрезанного изображения
        with rasterio.open(output_image_path, "w", **out_meta) as dest:
            dest.write(out_image)  # Записываем все каналы

        print(f"Обрезанное изображение сохранено: {output_image_path}")

def crop_image_by_lat_lon(product_folder, lat, lon, img_size=640):
    r10m_folders = find_channeles_folder(product_folder)
    for r10m_folder in r10m_folders:
        input_image_path = find_TCI_path(r10m_folder)
        output_image_folder = os.path.join(product_folder, "cropped_by_lat_lon")
        file_name = f'lat-{lat}_lon-{lon}-tile.jp2'
        output_image_path = os.path.join(output_image_folder, file_name)
        os.makedirs(os.path.dirname(output_image_path), exist_ok=True)
        with rasterio.open(input_image_path) as src:
            raster_crs = src.crs
            print(f"Растровая система координат: {raster_crs}")
            # Преобразование координат из EPSG:4326 в систему координат растра
            transformer = Transformer.from_crs(CRS.from_epsg(4326), raster_crs, always_xy=True)
            x_center, y_center = transformer.transform(lon, lat)
            # Проверка, что центр попадает внутрь растра
            if not (
                    src.bounds.left <= x_center <= src.bounds.right and src.bounds.bottom <= y_center <= src.bounds.top):
                raise ValueError("Указанный центр вне области растрового изображения.")
            # Размер стороны квадрата в пикселях
            square_size = img_size
            half_size = square_size // 2

            # Определяем индексы пикселей для обрезки
            row_start = int(src.height - (y_center - src.bounds.bottom) / src.res[1] - half_size)
            row_stop = row_start + square_size
            col_start = int((x_center - src.bounds.left) / src.res[0] - half_size)
            col_stop = col_start + square_size

            # Корректируем границы, чтобы они не выходили за пределы растра
            row_start_clipped = max(0, row_start)
            row_stop_clipped = min(src.height, row_stop)
            col_start_clipped = max(0, col_start)
            col_stop_clipped = min(src.width, col_stop)

            # Создаем окно обрезки
            window = Window(col_start_clipped, row_start_clipped,
                            col_stop_clipped - col_start_clipped,
                            row_stop_clipped - row_start_clipped)

            # Читаем данные из растра
            out_image = src.read(window=window)

            # Создаем новое изображение размером 640x640, заполненное черным цветом
            full_image = np.zeros((src.count, square_size, square_size), dtype=src.dtypes[0])

            # Вычисляем смещение для вставки обрезанного фрагмента
            row_offset = max(0, -row_start)
            col_offset = max(0, -col_start)

            # Вставляем обрезанный фрагмент в новое изображение
            full_image[:, row_offset:row_offset + out_image.shape[1],
            col_offset:col_offset + out_image.shape[2]] = out_image

            # Вычисляем новую матрицу трансформации
            new_transform = rasterio.transform.from_bounds(
                src.bounds.left + col_start * src.res[0],
                src.bounds.bottom + (src.height - row_stop) * src.res[1],
                src.bounds.left + col_stop * src.res[0],
                src.bounds.bottom + (src.height - row_start) * src.res[1],
                width=square_size,
                height=square_size
            )

            # Копируем метаданные
            out_meta = src.meta.copy()
            out_meta.update({
                "driver": "GTiff",  # Формат выходного файла
                "height": square_size,  # Высота обрезанного изображения
                "width": square_size,  # Ширина обрезанного изображения
                "transform": new_transform,  # Новая матрица трансформации
                "count": src.count,  # Число каналов (должно быть 3 для RGB)
                "dtype": src.dtypes[0]  # Тип данных (например, uint8)
            })

            # Сохраняем обрезанное изображение
            with rasterio.open(output_image_path, "w", **out_meta) as dest:
                dest.write(full_image)

            print(f"Тайл сохранен: {output_image_path}")
            return output_image_path

def remove_black_borders(product_folder, nodata_value=0):
    save_path = os.path.join(product_folder, "cropped")
    input_path = os.path.join(product_folder, 'cropped', 'cropped.jp2')
    output_path = os.path.join(save_path, 'cropped_blacked.jp2')
    os.makedirs(os.path.dirname(output_path), exist_ok=True)
    with rasterio.open(input_path) as src:
        # Чтение данных растра
        image = src.read()
        height, width = src.height, src.width
        transform = src.transform

        # Поиск непустых пикселей
        mask = (image != nodata_value).any(axis=0)  # Маска непустых пикселей
        rows = mask.any(axis=1)  # Непустые строки
        cols = mask.any(axis=0)  # Непустые столбцы

        # Определение границ непустой области
        row_start, row_end = rows.argmax(), len(rows) - rows[::-1].argmax()
        col_start, col_end = cols.argmax(), len(cols) - cols[::-1].argmax()

        # Создание окна обрезки
        window = Window(col_start, row_start, col_end - col_start, row_end - row_start)

        # Обрезка данных и обновление трансформации
        cropped_image = src.read(window=window)
        updated_transform = rasterio.windows.transform(window, transform)

        # Обновление метаданных
        out_meta = src.meta.copy()
        out_meta.update({
            "height": cropped_image.shape[1],
            "width": cropped_image.shape[2],
            "transform": updated_transform,
            "nodata": nodata_value
        })

        # Сохранение обрезанного изображения
        with rasterio.open(output_path, "w", **out_meta) as dest:
            dest.write(cropped_image)

    print(f"Обработка завершена. Результат сохранен в {output_path}")
    return output_path


def split_jp2_into_tiles(product_folder, tile_size=640):
    input_jp2_path = os.path.join(product_folder, 'cropped', 'cropped_blacked.jp2')
    output_dir = os.path.join(product_folder, 'tiles')
    name_of_tiles = os.path.basename(os.path.normpath(product_folder)) # name_of_tiles - название поседней папки
    # Создаем выходную директорию, если она не существует
    os.makedirs(output_dir, exist_ok=True)

    # Открываем JP2 файл
    with rasterio.open(input_jp2_path) as src:
        width, height = src.width, src.height
        transform = src.transform
        crs = src.crs

        # Создаем трансформер для преобразования в WGS84
        transformer = Transformer.from_crs(crs, "EPSG:4326", always_xy=True)

        # Вычисляем новые размеры для кратности tile_size
        new_width = ((width + tile_size - 1) // tile_size) * tile_size
        new_height = ((height + tile_size - 1) // tile_size) * tile_size

        # Если размеры уже кратны tile_size, ничего не меняем
        if new_width == width and new_height == height:
            padded_data = src.read()
            updated_transform = transform
        else:
            # Читаем исходные данные
            data = src.read()

            # Создаем массив для расширенного изображения
            padded_data = np.zeros((data.shape[0], new_height, new_width), dtype=data.dtype)

            # Копируем исходные данные в новый массив
            padded_data[:, :height, :width] = data

            # Обновляем матрицу трансформации
            updated_transform = rasterio.Affine(
                transform.a, transform.b, transform.c,
                transform.d, transform.e, transform.f
            )

        # Вычисляем общее количество блоков
        total_blocks = (height // tile_size + (height % tile_size > 0)) * \
                       (width // tile_size + (width % tile_size > 0))
        # Проходим по всем блокам с помощью tqdm для отображения прогресса
        block_count = 0
        # Проходим по всем блокам с помощью tqdm для отображения прогресса
        with tqdm(total=total_blocks, desc="Обработка блоков", unit="блок") as pbar:
            for row in range(0, height, tile_size):
                for col in range(0, width, tile_size):
                    block_count += 1
                    # Определяем размер текущего блока
                    window = Window(col, row, min(tile_size, new_width - col), min(tile_size, new_height - row))

                    # Читаем данные блока
                    tile_data = padded_data[:, window.row_off:window.row_off + window.height,
                                window.col_off:window.col_off + window.width]

                    # Получаем географические координаты верхнего левого угла блока
                    ul_x, ul_y = rasterio.transform.xy(updated_transform, row, col)

                    # Преобразуем координаты в WGS84
                    wgs84_x, wgs84_y = transformer.transform(ul_x, ul_y)

                    # Формируем имя файла с координатами
                    filename = f"tile_{name_of_tiles}_{wgs84_x:.6f}_{wgs84_y:.6f}.jp2"
                    output_path = os.path.join(output_dir, filename)

                    # Копируем метаданные
                    out_meta = src.meta.copy()
                    out_meta.update({
                        "driver": "GTiff",
                        "height": window.height,
                        "width": window.width,
                        "transform": rasterio.windows.transform(window, updated_transform),
                    })

                    # Сохраняем блок
                    with rasterio.open(output_path, "w", **out_meta) as dst:
                        dst.write(tile_data)

                    # Обновляем прогресс-бар
                    pbar.update(1)

                # print(f"Сохранен блок: {output_path}")
    print("Разбиение завершено.")
    return output_dir

if __name__ == "__main__":
    product_folder = '../data/2025/01/10/S2B_T43PDN_MSIL2A_05_21_09'
    filename = '../polygon_files/india_abma.geojson'
    polygon_coords = read_polygon(filename)
    crop_image_by_polygon(product_folder, polygon_coords)
    final_cropped_image = remove_black_borders(product_folder)
    resample(final_cropped_image)
    split_jp2_into_tiles(product_folder)
